#!/bin/sh -f
# Configure wmii

# Configuration Variables
# I use Windows key by default.  Hello.  --Alexey Tourbin
MODKEY=Mod4


# Colors tuples: "<text> <background> <border>"
WMII_NORMCOLORS='#222200 #779977 #224466'
WMII_FOCUSCOLORS='#ddddbb #449944 #667744'

WMII_BACKGROUND='#006010'
WMII_FONT='-*-terminus-medium-*-*-*-14-*-*-*-*-*-koi8-r'

set -- $(echo $WMII_NORMCOLORS $WMII_FOCUSCOLORS)
#WMII_MENU="dmenu -font $WMII_FONT -normfg $1 -normbg $2 -selfg $4 -selbg $5"
WMII_MENU="dmenu -b -fn $WMII_FONT -nf $1 -nb $2 -sf $4 -sb $5"
WMII_9MENU="wmii9menu -font $WMII_FONT -nf $1 -nb $2 -sf $4 -sb $5 -br $6"
WMII_TERM="Terminal"

# Column Rules
wmiir write /colrules <<!
/.*/ -> 58+42
!

# Tagging Rules
wmiir write /tagrules <<!
/Thunderbird.*:.*/ -> 5
/Claws.*/ -> 5
/Licq.*/ -> ~
/Licq.*/ -> 4
/Psi.*/ -> ~
/Psi.*/ -> 4
/wmMoonClock.*/ -> ~
/wmMoonClock.*/ -> 4
/Gimp.*/ -> ~
/Beep.*/ -> ~
/xine.*/ -> ~
/xine.*/ -> 8
/XMMS.*/ -> ~
/MPlayer.*/ -> ~
/.*/ -> !
/.*/ -> 1
!

# Status Bar Info
status() {
	echo -n $(date) $(uptime | sed 's/.*://; s/,//g') $(free -k -o | grep "Mem:" | awk '{print $3"k / "$4"k"}')
}

# MPD status Bar Info
mpdstatus() {
	echo -n $(mpc | sed 's/repeat.*//g')
}

# Volume Control

volume() {
	mpc volume $1"1"; sleep 0.2
	mpc volume $1"1"; sleep 0.2
	mpc volume $1"1"; sleep 0.2
	mpc volume $1"1"; sleep 0.2
	mpc volume $1"1"; sleep 0.2
}

# Event processing
#  processed later by `wmiiloop' and evaled
eventstuff() {
	cat <<'!'
	# Events
	Event Start
		case "$1" in
		wmiirc)
			exit;
		esac
	Event Key
		fn=$(echo $@ | sed 's/[^a-zA-Z_0-9]/_/g')
		Key_$fn $@
	Event CreateTag
		echo "$WMII_NORMCOLORS" "$@" | wmiir create "/lbar/$@"
	Event DestroyTag
		wmiir remove "/lbar/$@"
	Event FocusTag
		wmiir xwrite "/lbar/$@" "$WMII_FOCUSCOLORS" "$@"
	Event UnfocusTag
		wmiir xwrite "/lbar/$@" "$WMII_NORMCOLORS" "$@"
	Event UrgentTag
		shift
		wmiir xwrite "/lbar/$@" "*$@"
	Event NotUrgentTag
		shift
		wmiir xwrite "/lbar/$@" "$@"
	Event LeftBarClick
		shift
		wmiir xwrite /ctl view "$@"
	Event RightBarClick
		button=$1; bar=$2
		# if [ "X$bar" = "Xmpdstatus" ]; then
			case "$button" in
				1) mpc toggle ;;
				2) mpc prev ;;
				3) mpc next ;;
				4) volume "+" & ;;
				5) volume "-" & ;;
			esac
		# fi
	# Actions
	Action quit
		wmiir xwrite /ctl quit
	Action rehash
		proglist $PATH >$progsfile
	Action status
		set +xv
		if wmiir remove /rbar/status 2>/dev/null; then
			sleep 2
		fi
		echo "$WMII_NORMCOLORS" | wmiir create /rbar/status
		while status | wmiir write /rbar/status; do
			sleep 1
		done
	Action mpdstatus
		if wmiir remove /rbar/mpdstatus 2>/dev/null; then
			sleep 2
		fi
		echo "$WMII_NORMCOLORS" | wmiir create /rbar/mpdstatus
		while mpdstatus | wmiir write /rbar/mpdstatus; do
			sleep 1
		done		
	Event ClientMouseDown
		client=$1; button=$2
		case "$button" in
		3)
			do=$($WMII_9MENU -initial "${menulast:-SomeRandomName}" Nop Delete)
			case "$do" in
			Delete)
				wmiir xwrite /client/$client/ctl kill
			esac
			menulast=${do:-"$menulast"}
		esac
	# Key Bindings
	Key $MODKEY-Control-t
		case $(wmiir read /keys | wc -l | tr -d ' \t\n') in
		0|1)
			echo -n $Keys | tr ' ' '\012' | wmiir write /keys
			wmiir xwrite /ctl grabmod $MODKEY;;
		*)
			wmiir xwrite /keys $MODKEY-Control-t
			wmiir xwrite /ctl grabmod Mod3;;
		esac
	Key $MODKEY-h
		wmiir xwrite /tag/sel/ctl select left
	Key $MODKEY-l
		wmiir xwrite /tag/sel/ctl select right
	Key $MODKEY-j
		wmiir xwrite /tag/sel/ctl select down
	Key $MODKEY-k
		wmiir xwrite /tag/sel/ctl select up
	Key $MODKEY-space
		wmiir xwrite /tag/sel/ctl select toggle
	Key $MODKEY-d
		wmiir xwrite /tag/sel/ctl colmode sel default
	Key $MODKEY-s
		wmiir xwrite /tag/sel/ctl colmode sel stack
	Key $MODKEY-m
		wmiir xwrite /tag/sel/ctl colmode sel max
	Key $MODKEY-a
		Action $(actionlist | $WMII_MENU) &
	Key $MODKEY-p
		sh -c "$($WMII_MENU <$progsfile)" &
	Key $MODKEY-t
		wmiir xwrite /ctl "view $(tagsmenu)" &
	Key $MODKEY-Return
		$WMII_TERM &
	Key $MODKEY-Shift-h
		wmiir xwrite /tag/sel/ctl send sel left
	Key $MODKEY-Shift-l
		wmiir xwrite /tag/sel/ctl send sel right
	Key $MODKEY-Shift-j
		wmiir xwrite /tag/sel/ctl send sel down
	Key $MODKEY-Shift-k
		wmiir xwrite /tag/sel/ctl send sel up
	Key $MODKEY-Shift-space
		wmiir xwrite /tag/sel/ctl send sel toggle
	Key $MODKEY-Shift-c
		wmiir xwrite /client/sel/ctl kill
	Key $MODKEY-Shift-t
		wmiir xwrite "/client/$(wmiir read /client/sel/ctl)/tags" "$(tagsmenu)" &
	Key XF86AudioRaiseVolume
		volume "+" &
	Key XF86AudioLowerVolume
		volume "-" &
	Key XF86AudioPrev
		mpc prev
	Key XF86AudioNext
		mpc next
	Key XF86AudioPlay
		mpc toggle
	Key XF86AudioStop
		mpc stop
	Key XF86Standby
		/usr/bin/xscreensaver-command -lock
	Key XF86HomePage
		/usr/bin/firefox &
	Key XF86Mail
		/usr/bin/claws-mail &
	Key XF86Forward
		/usr/bin/licq &
	Key XF86Back
		/usr/bin/psi &
	Key Print
		import -w root screenshot--`date +%F--%T`.jpg
!
	for i in 0 1 2 3 4 5 6 7 8 9; do
		cat <<!
	Key $MODKEY-$i
		wmiir xwrite /ctl view "$i"
	Key $MODKEY-Shift-$i
		wmiir xwrite /client/sel/tags "$i"
!
	done
}

# WM Configuration
wmiir write /ctl << EOF
font $WMII_FONT
focuscolors $WMII_FOCUSCOLORS
normcolors $WMII_NORMCOLORS
grabmod $MODKEY
border 0
EOF

# Feed events to `wmiiloop' for processing
IFS=''
eval $(eventstuff | sed "s/\\\$MODKEY/$MODKEY/g;s/^[	]//" | wmiiloop)
unset IFS

# Functions
Action() {
	action=$1; shift
	if [ -n "$action" ]; then
		Action_$action $@ \
		|| conf_which $action $@
	fi
}

proglist() {
	paths=$(echo "$@" | sed 'y/:/ /')
	ls -lL $paths 2>/dev/null \
		| awk '$1 ~ /^[^d].*x/ { print $NF }' \
		| sort | uniq
}

# Misc
progsfile="$WMII_NS_DIR/.proglist"
Action status &
Action mpdstatus &
~/.xsmute.pl &
proglist $PATH >$progsfile &

xsetroot -solid "$WMII_BACKGROUND" &

export WMII_MENU WMII_9MENU WMII_FONT WMII_TERM
export WMII_FOCUSCOLORS WMII_SELCOLORS WMII_NORMCOLORS

# Setup Tag Bar
wmiir ls /lbar |
while read bar; do
	wmiir remove "/lbar/$bar"
done
seltag="$(wmiir read /tag/sel/ctl 2>/dev/null)"
wmiir ls /tag | sed -e 's|/||; /^sel$/d' |
while read tag; do
	if [ "X$tag" = "X$seltag" ]; then
		echo "$WMII_FOCUSCOLORS" "$tag" | wmiir create "/lbar/$tag" 
	else
		echo "$WMII_NORMCOLORS" "$tag" | wmiir create "/lbar/$tag"
	fi
done

# More functions
tagsmenu() {
        wmiir ls /tag | sed 's|/||; /^sel$/d' | $WMII_MENU
}

actionlist() {
	{	proglist $WMII_CONFPATH
		echo -n $Actions | tr ' ' '\012'
	} | sort | uniq
}

conf_which() {
	which=$(which which)
	prog=$(PATH="$WMII_CONFPATH" $which $1)
	shift
	if [ -n "$prog" ]; then
		$prog
	fi
}

# Stop any running instances of wmiirc
echo Start wmiirc | wmiir write /event || exit 1

wmiir read /event |
while read event; do
	set -- $event
	event=$1; shift
	Event_$event $@
done 2>/dev/null
